# -*- coding: utf-8 -*-
import json

import autocomplete_light
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from django.template.response import TemplateResponse
from django.core import serializers
from django.conf.urls import patterns, url

from jmb.core.shortcuts import render_to_string
from jmb.jadmin import JumboModelAdmin
from jmb.ticket import utils, models
from . import forms as ticket_forms
from .ticket import TicketAdmin  # noqa export for other packages
from .plan import PlanAdmin  # noqa idem
from .forms import TicketProjectAdminForm  # noqa idem

#########################################
#              CATEGORY
#########################################
class CategoryTranslationInline(admin.TabularInline):
    model = models.CategoryTranslation
    extra = 0
    fields = ('language', 'name',)

class CategoryAdmin(JumboModelAdmin):
    fieldsets = (
        (_('general admin'), {'fields': ('name', 'description', 'code')}),
    )
    list_display = (
        'date_create', 'name', 'code',
        'get_edit_icon', 'get_delete_icon'
    )
    list_editable = ('code',)
    inlines = [CategoryTranslationInline, ]


#########################################
#              TYPOLOGY
#########################################
class TypologyTranslationInline(admin.TabularInline):
    model = models.TypologyTranslation
    extra = 0
    fields = ('language', 'name',)

class TypologyAdmin(JumboModelAdmin):
    fieldsets = (
        (_('general admin'), {'fields': ('name', 'description',)}),
    )
    list_display = (
        'date_create', 'description', 'name',
        'get_edit_icon', 'get_delete_icon'
    )
    inlines = [TypologyTranslationInline, ]


class TicketProjectAdmin(JumboModelAdmin):
    form = ticket_forms.TicketProjectAdminForm
    fieldsets = (
        (_('general admin'), {'fields': (
            ('name', 'status',),
            ('organization', 'address'),
            'description',
            ('managers', 'assistance_email'),
            ('staff', 'customers'),
            ('date_start', 'date_end'),
        )}),
    )
    filter_horizontal = ('managers', 'staff', 'customers')
    list_display = (
        'status', 'date_create', 'date_start', 'date_end',
        'organization', 'name', 'creator', 'description',
        'get_string_managers', 'get_string_staff', 'get_string_customers',
        'assistance_email',
        'get_edit_icon', 'get_delete_icon'
    )

    advanced_search_fields = (
        ('name__icontains:project', 'organization__name__icontains:organization'),
        ('description__icontains',),
        ('date_create__gte', 'date_create__lte',),
        ('date_start__gte', 'date_start__lte',),
        ('managers', 'staff')
    )
    action_form = ticket_forms.ProjectActionForm
    actions = ['create_bulk_tickets']

    def change_view(self, request, object_id, form_url='', extra_context=None):

        opts = self.model._meta
        app_label = opts.app_label

        self.change_form_template = [
            'admin/%s/%s/change_form.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_form.html' % app_label,
            'admin/change_form.html'
        ]

        jpopup_param = request.GET.get('jpopup', False)

        if request.method == 'POST':
            if jpopup_param:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                response = super(TicketProjectAdmin, self).change_view(
                    request, object_id, form_url, extra_context)

                _json = request.POST.get('_json', False)
                if _json:
                    response['location'] = reverse(
                        'admin:ticket_ticketproject_iframe', args=[object_id])
                return response

            response = super(TicketProjectAdmin, self).change_view(
                request, object_id, form_url, extra_context)
            return response

        if request.method == 'GET':
            if jpopup_param:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.change_form_template = 'admin/ticket/change_form_iframe.html'
        try:
            return super(TicketProjectAdmin, self).change_view(
                request, object_id, form_url, extra_context)
        except ticket_forms.ForbiddenException:
            return HttpResponseForbidden(render_to_string('jmb/forbidden.html'))

    def create_bulk_tickets(self, request, queryset):
        form = self.get_action_form_instance(request)
        kw = {
            'title':       form.cleaned_data['title'],
            'description': form.cleaned_data['description'],
            'typology':    form.cleaned_data['typology'],
            'category':    form.cleaned_data['category'],
            'priority':    form.cleaned_data['priority'],
            'date_create': form.cleaned_data['date'],
            'status':      form.cleaned_data['status'],
            'submitter':   utils.get_current_contact()[1],
        }
        for prj in queryset:
            tk = models.Ticket(project=prj, **kw)
            tk.save()
            for worker in form.cleaned_data['workers']:
                tk.assigned_to.add(worker)
            prj.save()

    def get_urls(self):
        urls = super(TicketProjectAdmin, self).get_urls()
        my_urls = patterns('',
                           url(
                               r'^iframe/(?P<object_id>\d+)/$',
                               self.iframe,
                               name='ticket_ticketproject_iframe'
                           ),
                           )
        return my_urls + urls

    def iframe(self, request, object_id):
        qs = super(TicketProjectAdmin, self).get_queryset(request).filter(id=object_id)
        data_json = serializers.serialize(
            "json", qs.all(), use_natural_keys=True)
        return TemplateResponse(request, 'admin/ticket/iframe_callback.html',
                                {"data_json": data_json}, status=200)


class TicketAssistanceAdmin(JumboModelAdmin):

    class Media:
        js = (
            "ticket/js/admin_dajaxice.js",
        )

    form = ticket_forms.TicketAssistanceAdminForm
    fieldsets = (
        (_('General Information'), {
            'fields': (
                'ticket', 'worker',
                ('date_begin',), ('date_end',),
                'job_description',
                ('work_duration', 'extra_duration', 'travel_duration',),
                ('kilometers', 'carrying', 'overnight_stay', 'procedure',)
            )
        }),
    )
    list_display = (
        'get_title', 'get_ticket_status_icon',
        'worker', 'get_project', 'get_organization',
        'date_begin', 'date_end', 'procedure',
        'get_edit_icon', 'get_delete_icon'
    )
    advanced_search_fields = (
        ('ticket__title__icontains:ticket', 'worker__last_name__icontains',
         'worker__first_name__icontains'),
        ('ticket__category__in:category', 'ticket__typology__in:typology',
         'ticket__priority__in:priority'),
        ('ticket__project__name__icontains:project', 'ticket__organization__name__icontains'),
        ('ticket__description__icontains', 'job_description__icontains',
         'ticket__project__organization__addresses__name__icontains:indirizzo',),
        ('ticket__project__organization__addresses__zip_code:cap',),
        ('date_begin__gte', 'date_end__lte', 'date_begin__range'),
    )

    def get_title(self, obj):
        href = u'<a class="iframe" href="{url}?_popup=1&title=1&nobutton=1">{title}</a>'
        return href.format(
            url='%s%s/detail/' % (reverse('admin:ticket_ticket_changelist'), obj.ticket.id),
            title=obj.ticket.title,
        )
    get_title.allow_tags = True
    get_title.short_description = _("Title")

    def get_ticket_status_icon(self, obj):
        return """<img src='/static/ticket/img/ticket_status_%(status)s.png'
            alt='%(window_title)s' title='%(window_title)s'/>""" % {
            'status': obj.ticket.status,
            'window_title': 'Ticket %s' % str(obj.ticket)
        }
    get_ticket_status_icon.allow_tags = True
    get_ticket_status_icon.short_description = _("ST")

    def get_project(self, obj):
        return obj.ticket.project or ''
    get_project.short_description = _("Project")

    def get_organization(self, obj):
        organization = obj.ticket.organization
        if not organization and obj.ticket.project:
            organization = obj.ticket.project.organization
        return organization or ''
    get_organization.short_description = _("Organization")

    def get_form(self, request, obj=None, **kwargs):
        form = super(TicketAssistanceAdmin, self).get_form(request, obj, **kwargs)
        # se la richiesta ha un ticket predefinito
        if obj is None and request.GET.get("ticket", False):
            ticket_id = request.GET.get("ticket")
        # se è in edit
        elif obj and obj.ticket:
            ticket_id = obj.ticket.id
        else:
            ticket_id = None

        # se la richiesta ha un plan
        if obj is None and request.GET.get("plan", False):
            plan = models.Plan.objects.get(id=request.GET.get("plan"))
            ticket_id = plan.ticket.id
            worker_choices = [(w.id, str(w)) for w in plan.workers.all()]
            if worker_choices:
                form.base_fields['worker'].choices = worker_choices
            form.base_fields['ticket'].initial = plan.ticket_id
            form.base_fields['date_end'].initial = plan.date_end
            form.base_fields['date_begin'].initial = plan.date_begin
            return form
        # se la richiesta ha un ticket
        if obj is None and request.GET.get("ticket", False):
            ticket = models.Ticket.objects.get(id=request.GET.get("ticket"))
            ticket_id = ticket.id
            worker_choices = [(w.id, str(w)) for w in ticket.project.staff.all()]
            if worker_choices:
                form.base_fields['worker'].choices = worker_choices
            form.base_fields['ticket'].initial = ticket_id
            # form.base_fields['date_end'].initial = ticket.date_end
            # form.base_fields['date_begin'].initial = ticket.date_begin
            return form

        if ticket_id:
            ticket = models.Ticket.objects.get(id=ticket_id)
            user, contact = utils.get_current_contact(request.user)
            if contact:
                role = contact.get_role(ticket.organization, ticket.project)
                if role == "staff":
                    get = request.GET.copy()
                    get['worker'] = contact.id
                    request.GET = get
                    get = request.GET.copy()
                    form.base_fields['worker'].choices = [(contact.id, str(contact))]
            return form

        return form

    def delete_view(self, request, object_id, extra_context=None):

        if request.method == 'DELETE':
            try:
                instance = models.TicketAssistance.objects.get(id=object_id)
                instance.delete()
            except ObjectDoesNotExist:
                raise Http404
            response = HttpResponse('', content_type="application/json", status=200)
        else:
            response = super(TicketAssistanceAdmin, self).delete_view(
                request, object_id, extra_context)
        return response

    def get_urls(self):
        urls = super(TicketAssistanceAdmin, self).get_urls()
        my_urls = patterns('',
                           url(
                               r'^only_result_list/$',
                               self.only_result_list_view,
                               name='ticket_ticketassistance_only_result_list'
                           ),
                           url(
                               r'^iframe/(?P<object_id>\d+)/$',
                               self.iframe,
                               name='ticket_assistances_iframe'
                           ),
                           )
        return my_urls + urls

    def iframe(self, request, object_id):
        qs = super(TicketAssistanceAdmin, self).get_queryset(request).filter(id=object_id)
        data_json = serializers.serialize(
            "json", qs.all(), extras=('label'), use_natural_keys=True)
        return TemplateResponse(request, 'admin/ticket/iframe_callback.html',
                                {"data_json": data_json}, status=200)

    def changelist_view(self, request, extra_context=None):
        opts = self.model._meta
        app_label = opts.app_label

        self.change_list_template = [
            'admin/%s/%s/change_list.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_list.html' % app_label,
            'admin/change_list.html'
        ]

        json_param = request.GET.get('_json', False)
        if json_param:
            qs = super(TicketAssistanceAdmin, self).get_queryset(request)
            qs = qs.select_related('ticket', 'ticket__project', 'ticket__project__organization')
            results = serializers.serialize("json", qs.all(), extras=('label',),
                                            use_natural_keys=True)
            return HttpResponse(results, content_type="application/json")
        else:
            response = super(TicketAssistanceAdmin, self).changelist_view(request, extra_context)
        return response

    def change_view(self, request, object_id, form_url='', extra_context=None):

        opts = self.model._meta
        app_label = opts.app_label

        self.change_form_template = [
            'admin/%s/%s/change_form.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_form.html' % app_label,
            'admin/change_form.html'
        ]

        if request.method == 'POST':
            json_param = request.GET.get('json', False)
            if json_param:
                try:
                    request_dict = json.loads(request.body)
                    instance = models.TicketAssistance.objects.get(id=object_id)
                except ValueError:
                    return HttpResponse('Invalid Request', status=400)
                except ObjectDoesNotExist:
                    raise Http404

                form = TicketAssistanceAdmin.form(request_dict, instance=instance)
                if form.is_valid():
                    json_data = serializers.serialize("json", [instance, ], extras=('label',))
                    return HttpResponse(json_data, content_type="application/json",
                                        status=200)
                else:
                    return HttpResponse(form.errors.items(), status=400)

            jpopup_param = request.GET.get('jpopup', False)
            if jpopup_param:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.change_form_template = 'admin/ticket/change_form_iframe.html'
                response = super(TicketAssistanceAdmin, self).change_view(
                    request, object_id, form_url, extra_context)

            _json = request.POST.get('_json', False)
            if _json:
                response['location'] = reverse('admin:ticket_assistances_iframe', args=[object_id])

            try:
                return response
            except UnboundLocalError:
                return super(TicketAssistanceAdmin, self).change_view(
                    request, object_id, form_url, extra_context)

        if request.method == 'GET':

            json_param = request.GET.get('json', False)
            jpopup_param = request.GET.get('jpopup', False)

            if json_param:
                qs = super(TicketAssistanceAdmin, self).get_queryset(request).filter(id=object_id)
                data = serializers.serialize("json", qs.all(), extras=(
                    'label',), use_natural_keys=True)
                return HttpResponse(data, content_type="application/json")
            if jpopup_param:
                try:
                    instance = models.TicketAssistance.objects.get(id=object_id)
                except ObjectDoesNotExist:
                    raise Http404
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.change_form_template = 'admin/ticket/change_form_iframe.html'
        try:
            return super(TicketAssistanceAdmin, self).change_view(
                request, object_id, form_url, extra_context)
        except ticket_forms.ForbiddenException:
            return HttpResponseForbidden(render_to_string('jmb/forbidden.html'))

    def add_view(self, request, form_url='', extra_context=None):

        opts = self.model._meta
        app_label = opts.app_label

        self.add_form_template = [
            'admin/%s/%s/change_form.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_form.html' % app_label,
            'admin/change_form.html'
        ]

        jpopup_param = request.GET.get('jpopup', False)
        jpopup_param2 = request.GET.get('_popup', False)

        if request.method == 'POST':
            if jpopup_param or jpopup_param2:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.add_form_template = 'admin/ticket/change_form_iframe.html'

        if request.method == 'GET':
            if jpopup_param or jpopup_param2:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.add_form_template = 'admin/ticket/change_form_iframe.html'
        try:
            return super(TicketAssistanceAdmin, self).add_view(request, form_url, extra_context)
        except ticket_forms.ForbiddenException:
            return HttpResponseForbidden(render_to_string('jmb/forbidden.html'))

    def response_add(self, request, obj, post_url_continue=None):
        method = '_json_continue' if '_json_continue' in request.POST else '_json'
        if request.GET.get("_refresh"):
            return HttpResponseRedirect(
                reverse('admin:ticket_ticket_json',
                        args=[obj.ticket.pk]) + "?_refresh=1&_method=" + method)
        if '_json' in request.POST:
            return HttpResponseRedirect(reverse('admin:ticket_assistances_iframe', args=[obj.id]))
        return super(TicketAssistanceAdmin, self).response_add(request, obj, post_url_continue)

    def only_result_list_view(self, request, extra_context=None):
        opts = self.model._meta
        app_label = opts.app_label

        self.change_list_template = ('admin/%s/only_result_list.html' % 
                                     (app_label))

        return super(TicketAssistanceAdmin, self).changelist_view(request, extra_context)

class PriorityAdmin(JumboModelAdmin):
    fieldsets = (
        (_('General Information'), {
            'fields': (
                'priority', 'name',
                ('color', 'color_text',),
            )
        }),
    )
    list_display = (
        'priority', 'name',
        'color', 'color_text', 'preview',
        'get_edit_icon', 'get_delete_icon'
    )

    def preview(self, obj):
        return """<div style='background-color:%s'>
                <span style="color:%s">%s</span></div>""" % (
            obj.color, obj.color_text, _('Ticket')
        )

    class Media:
        js = (
            "ticket/js/colorpicker.js",
            "ticket/js/priority_color_picker.js",
        )
        css = {
            "all": ("ticket/css/colorpicker.css",)
        }

    def changelist_view(self, request, extra_context=None):
        if request.method == 'GET':
            json_param = request.GET.get('_json', False)
            if json_param:
                qs = super(PriorityAdmin, self).get_queryset(request)
                data = serializers.serialize("json", qs.all())
                return HttpResponse(data, content_type="application/json")
        return super(PriorityAdmin, self).changelist_view(request, extra_context)

    preview.allow_tags = True
    preview.short_description = _("Preview")

class TicketPermAdmin(JumboModelAdmin):
    form = autocomplete_light.modelform_factory(models.TicketPerm, exclude={})
    list_editable = ('project_manager', 'project_staff',)
    list_display = ('contact', 'project_manager', 'project_staff',)
    search_fields = ('contact__first_name', 'contact__last_name')
    advanced_search_fields = (
        ('contact__last_name', 'contact__role',),
        ('project_manager', 'project_staff'),
    )

admin.site.register(models.TicketAssistance, TicketAssistanceAdmin)
admin.site.register(models.Category, CategoryAdmin)
admin.site.register(models.TicketProject, TicketProjectAdmin)
admin.site.register(models.Typology, TypologyAdmin)
admin.site.register(models.Priority, PriorityAdmin)
admin.site.register(models.TicketPerm, TicketPermAdmin)
