# coding: utf-8
import datetime

import autocomplete_light
from autocomplete_light.widgets import ChoiceWidget, MultipleChoiceWidget
from bs4 import BeautifulSoup
from django import forms
from django.conf import settings
from django.db.models import Q
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.utils.html import escape, conditional_escape

from jmb.core.forms.widgets import TimePicker, DatePicker
from jmb.jadmin import JumboActionForm
from jmb.core.forms.models import AdminModelForm
from jmb.core.forms.fields import JumboDateField
from jmb.core.utils.ics import EmailMessageCalendar
from jmb.ticket.models import Organization, Contact, OrganizationAddress
from jmb.ticket import utils, models


class ForbiddenException(Exception):
    pass

class TicketAdminForm(AdminModelForm):

    def field_setattr(self, key, attr, value):
        if key in self.fields:
            setattr(self.fields[key], attr, value)

    def __init__(self, *args, **kwargs):

        super(TicketAdminForm, self).__init__(*args, **kwargs)
        organization_id = self.get_object_id('organization')
        project_id = self.get_object_id('project')

        organizations = Organization.customers_owners.filter(status=1)
        projects = models.TicketProject.objects.none()
        assigned_bys = Contact.objects.none()
        assigned_tos = Contact.objects.none()
        submitters = Contact.objects.none()
        self.fields['project'].queryset = projects
        self.fields['project'].required = False
        user, contact = utils.get_current_contact()

        instance = getattr(self, 'instance', None)

        if user.is_superuser or (contact and contact.is_site_manager()):
            organizations = Organization.customers_owners.filter(status=1)

            if organization_id:
                organization = organizations.get(id=organization_id)
                projects = organization.ticketprojects

                if project_id:
                    project = projects.get(id=project_id)
                    assigned_bys = project.all_managers.qs
                    assigned_tos = project.all_staff.qs
                    submitters = project.submitters.qs
                    self.fields['assigned_by'].initial = contact
                    self.fields['assigned_to'].initial = contact
                    self.fields['submitter'].initial = contact

        elif contact:
            organizations = contact.ticket_organizations
            if not organizations:
                raise ForbiddenException("No Perms")

            if organization_id:
                try:
                    organization = organizations.get(id=organization_id)
                except Organization.DoesNotExist:
                    raise ForbiddenException("No Perms")

                projects = contact.get_organization_ticketprojects(organization)

                if project_id:
                    try:
                        project = projects.get(id=project_id)
                    except models.TicketProject.DoesNotExist:
                        raise ForbiddenException("No perms for ticket %s" % project_id)
                    ticket_role = contact.get_role(organization, project)

                    if ticket_role and ticket_role == "manager":
                        assigned_bys = project.all_managers.qs
                        assigned_tos = project.all_staff.qs
                        submitters = project.submitters.qs
                        self.field_setattr('assigned_by', 'initial', contact)
                        self.field_setattr('assigned_to', 'initial', contact)
                        self.field_setattr('submitter', 'initial', contact)

                    elif ticket_role and ticket_role == "staff":
                        submitters = project.submitters.qs
                        if instance and instance.pk:
                            pass
                        else:
                            self.field_setattr('assigned_to', 'initial', contact)
                            self.field_setattr('submitter', 'initial', contact)

                    elif ticket_role and ticket_role == "customer":
                        submitters = project.submitters.qs
                        self.field_setattr('submitter', 'initial', contact)
                        self.field_setattr('public', 'initial', True)

                    else:
                        submitters = models.AllSet([contact]).qs

        self.fields['organization'] = forms.ModelChoiceField(
            label=_('Organization'),
            queryset=organizations, required=True,
            widget=autocomplete_light.ChoiceWidget('CustomerOwnerOrganization',)
        )
        self.fields['address'] = forms.ModelChoiceField(
            label=_('Address'),
            queryset=OrganizationAddress.objects, required=False,
            widget=ChoiceWidget('ProjectOrganizationAddress')
        )
        if (contact and Contact.owners.filter(id=contact.id).count() == 0):
            self.fields['organization'] = forms.ModelChoiceField(
                queryset=organizations, required=True
            )

        self.field_setattr('project', 'queryset', projects)
        self.field_setattr('assigned_by', 'queryset', assigned_bys)
        self.fields['assigned_to'] = forms.ModelMultipleChoiceField(
            queryset=assigned_tos, required=False,
            #            widget=autocomplete_light.MultipleChoiceWidget('WorkerContact')
        )
        self.field_setattr('submitter', 'queryset', submitters)

    def clean_public(self):
        user, contact = utils.get_current_contact()
        value = self.cleaned_data['public']

        if contact and not contact.is_owner():
            if not value:
                value = True

        return value

    def clean(self):
        cl = super(TicketAdminForm, self).clean()
        assigned_to_input = cl.get('assigned_to', None)
        assigned_by_input = cl.get('assigned_by', None)

        if assigned_to_input and not assigned_by_input:
            user, contact = utils.get_current_contact()
            if not contact and (user.is_superuser or (contact and contact.is_site_manager())):
                raise forms.ValidationError(_("assigned by field is required"))
            else:
                cl["assigned_by"] = contact

        if assigned_to_input and cl.get('progress_status') == 1:  # to_be_assigned
            cl["progress_status"] = 7  # assigned
        return cl


class ProjectActionForm(JumboActionForm):
    STATUS_CODES = (
        (0, _('Closed')),
        (1, _('Open')),
    )
    title = forms.CharField(required=False)
    date = JumboDateField(required=False, widget=DatePicker)
    description = forms.CharField(widget=forms.Textarea(attrs={'rows': 5, 'cols': 300}),
                                  required=False)
    typology = forms.ModelChoiceField(queryset=models.Typology.objects, required=False)
    category = forms.ModelChoiceField(queryset=models.Category.objects, required=False)
    priority = forms.ModelChoiceField(queryset=models.Priority.objects, required=False, initial=20)
    status = forms.ChoiceField(choices=STATUS_CODES, required=False)
    workers = forms.ModelMultipleChoiceField(
        queryset=Contact.objects, required=False, label=_('workers'),
        widget=autocomplete_light.MultipleChoiceWidget('Contact'),
    )

    fields_map = {
        'create_bulk_tickets': ['title', 'description', 'priority', 'typology',
                                'category', 'workers', 'date', 'status'],
        'export_action': ['output_type'],
        'junk': ['date', 'output_type', 'workers'],
    }

    class Meta:
        model = models.Ticket
        fields = "__all__" 
        exclude = ['output_type']

class TicketProjectAdminForm(AdminModelForm):

    def __init__(self, *args, **kwargs):
        super(TicketProjectAdminForm, self).__init__(*args, **kwargs)

        organization_id = self.get_object_id('organization')
        # project_id = self.get_object_id('project')

        organization_qs = Organization.customers_owners

        # imposta i queryset vuoti
        # project_qs = TicketProject.objects.none()
        manager_qs = Contact.objects.none()
        staff_qs = Contact.objects.none()
        customer_qs = Contact.objects.none()

        if organization_id:
            manager_qs = Contact.owners
            staff_qs = Contact.workers
            customer_qs = Contact.objects.filter(organization__id=organization_id)

        self.fields['organization'] = forms.ModelChoiceField(
            queryset=organization_qs,
            widget=ChoiceWidget('CustomerOwnerOrganization')
        )
        self.fields['managers'] = forms.ModelMultipleChoiceField(
            widget=MultipleChoiceWidget('OwnerContact'),
            queryset=manager_qs
        )
        self.fields['staff'] = forms.ModelMultipleChoiceField(
            widget=MultipleChoiceWidget('WorkerContact'),
            queryset=staff_qs
        )
        self.fields['customers'] = forms.ModelMultipleChoiceField(
            required=False,
            widget=MultipleChoiceWidget('CustomerContact'),
            queryset=customer_qs
        )
        self.fields['address'] = forms.ModelChoiceField(
            queryset=OrganizationAddress.objects, required=False,
            widget=ChoiceWidget('ProjectOrganizationAddress')
        )
        self.fields['description'] = forms.CharField(widget=forms.Textarea(attrs={
            'rows': '5', 'cols': '40', 'class': 'vLargeTextField'
        }))

    class Meta:
        model = models.TicketProject
        fields = "__all__" 
        
class PatchModelForm(forms.ModelForm):
    # snipper django: https://djangosnippets.org/snippets/2758/

    def __init__(self, *args, **kwargs):
        for fieldname in self.base_fields:
            self.base_fields[fieldname].required = False
        self.empty_permited = True
        return super(PatchModelForm, self).__init__(*args, **kwargs)

    def _clean_fields(self):
        for name, field in self.fields.items():
            # value_from_datadict() gets the data from the data dictionaries.
            # Each widget type knows how to retrieve its own data, because some
            # widgets split data over several HTML fields.
            value = field.widget.value_from_datadict(
                self.data,
                self.files,
                self.add_prefix(name)
            )
            # this is the key difference with the inherited behavior so fields
            # not present in the bound data do not get updated with None values

            if (self.add_prefix(name) not in self.data and
                    self.add_prefix(name) + "-autocomplete" not in self.data):
                continue
            try:
                if isinstance(field, forms.FileField):
                    initial = self.initial.get(name, field.initial)
                    value = field.clean(value, initial)
                else:
                    value = field.clean(value)
                self.cleaned_data[name] = value
                if hasattr(self, 'clean_%s' % name):
                    value = getattr(self, 'clean_%s' % name)()
                    self.cleaned_data[name] = value
            except forms.ValidationError, e:
                self._errors[name] = self.error_class(e.messages)
                if name in self.cleaned_data:
                    del self.cleaned_data[name]

class PlanAdminForm(PatchModelForm):

    workers = forms.ModelMultipleChoiceField(
        queryset=Contact.objects, required=False, label=_('workers'),
        widget=autocomplete_light.MultipleChoiceWidget('Contact')
    )
    ticket = forms.ModelChoiceField(
        queryset=models.Ticket.objects, required=True,
        widget=autocomplete_light.ChoiceWidget('TicketAutocomplete')
    )
    address = forms.ModelChoiceField(
        label=_('Address'),
        queryset=OrganizationAddress.objects, required=False,
        widget=ChoiceWidget('OrganizationAddress')
    )
    close_ticket = forms.BooleanField(label=_('Close') + " " + _('Ticket'),)

    # date_begin = forms.CharField(required=False)
    # date_end = forms.CharField(required=False)

    class Meta:
        fields = "__all__" 
        model = models.Plan

    def __init__(self, *args, **kwargs):
        super(PlanAdminForm, self).__init__(*args, **kwargs)
        # self.fields['workers'].widget.attrs['size']='10'
        self.fields['ticket'].required = True
        self.fields['description'] = forms.CharField(widget=forms.Textarea(attrs={
            'rows': '5', 'cols': '10', 'class': 'vLargeTextField'
        }), required=False, label=_('Job Description'))
        self.fields.keyOrder = ['ticket', 'workers', 'description', 'date_begin', 'date_end',
                                'address', 'status', 'priority', 'close_ticket']
        self.fields['date_begin'].required = True
        self.fields['date_end'].required = True
        instance = kwargs.pop("instance", False)
        if not instance or instance.ticket.status == 0:
            self.fields['close_ticket'] = forms.BooleanField(widget=forms.HiddenInput(),
                                                             required=False)

    def clean(self):

        return self.cleaned_data
        # CODICE SEGUENTE NON UTILIZZATO

        if 'workers' in self.cleaned_data:
            l = list(self.cleaned_data['workers'])
        else:
            l = []
        # SISTEMARE TODO
        if 'extraworkers' in self.cleaned_data:
            l.extend(list(self.cleaned_data['extraworkers']))

        self.cleaned_data['workers'] = l

        return self.cleaned_data

class TicketAssistanceAdminForm(PatchModelForm):

    worker = forms.ModelChoiceField(
        queryset=Contact.objects, required=True, label=_('workers'),
        widget=autocomplete_light.ChoiceWidget('Contact')
    )
    date_end = forms.TimeField(required=False, widget=TimePicker())

    class Meta:
        model = models.TicketAssistance
        fields = "__all__" 
        
    def clean(self):

        date_end = self.cleaned_data.get('date_end')
        if isinstance(date_end, datetime.time):
            self.cleaned_data['date_end'] = datetime.datetime.combine(
                self.cleaned_data['date_begin'], self.cleaned_data['date_end'])
        return self.cleaned_data

    def __init__(self, *args, **kwargs):
        super(TicketAssistanceAdminForm, self).__init__(*args, **kwargs)
        self.fields['date_end'].label = _("End")
        self.fields['date_begin'].label = _("Begin")
        self.fields['ticket'].required = True
        self.fields['worker'].required = True

class SelectMultipleWithDisable(forms.SelectMultiple):

    '''
    Widget di select multipla che gestisce le option disablitate
    disabled: [] lista con gli id da disabilitare
    '''

    def __init__(self, attrs=None, choices=(), disabled=()):
        super(SelectMultipleWithDisable, self).__init__(attrs)
        self.disabled = disabled
        self.choices = list(choices)

    def render_option(self, selected_choices, option_value, option_label):
        option_value = force_unicode(option_value)
        if option_value in selected_choices:
            selected_html = u' selected="selected"'
            if not self.allow_multiple_selected:
                selected_choices.remove(option_value)
        else:
            selected_html = ''
        if int(option_value) in self.disabled:
            disabled = ' disabled="disabled"'
        else:
            disabled = ''
        return u'<option %s value="%s"%s>%s</option>' % (
            disabled,
            escape(option_value), selected_html,
            conditional_escape(force_unicode(option_label)))


class InvitationForm(forms.Form):

    workers = forms.MultipleChoiceField()
    contacts = forms.ModelMultipleChoiceField(
        queryset=Contact.objects.exclude(email__isnull=True).exclude(email__exact=''),
        required=False,
        widget=autocomplete_light.MultipleChoiceWidget('ContactsEmailAutocomplete')
    )
    emails = forms.CharField(required=False)
#    content = forms.CharField(required=False, widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        self.object_id = kwargs.pop("object_id")
        plan = models.Plan.objects.get(id=self.object_id)
        allworkers = plan.workers.all()
        # lista tutti i workers
        workers = [(worker.id, worker.get_full_name()) for worker in allworkers]
        # lista workers senza email
        self.workers_disabled = [worker.id for worker in allworkers if not worker.email]
        super(InvitationForm, self).__init__(*args, **kwargs)
        self.fields['workers'].widget = SelectMultipleWithDisable(
            choices=workers, disabled=self.workers_disabled)
        self.fields['workers'].choices = workers
        self.fields['workers'].required = False

    def clean_workers(self):
        # imposta i workers come lista se viene passato solo un workers che non è una lista
        workers_input = self.cleaned_data['workers']
        if not isinstance(workers_input, list):
            workers_input = [workers_input]

        workers = Contact.objects.filter(id__in=workers_input)
        for worker in workers:
            if not worker.email:
                raise forms.ValidationError("Contact has not email")
        return workers_input

    def clean_emails(self):
        emails_input = self.cleaned_data['emails']
        emails = emails_input.split(" ")
        return emails

    def send(self, object_id):
        cd = self.cleaned_data
        user, current_contact = utils.get_current_contact()
        contacts = Contact.objects.filter(Q(id__in=cd['workers']) | Q(id__in=cd['contacts']))
        contacts_email = [contact.email for contact in contacts]
        recipient_list = contacts_email + cd['emails'] + [current_contact.email]
        recipient_list = set([email for email in recipient_list if email])

        plan = models.Plan.objects.get(id=object_id)

        summary = "%s %s" % (plan.description, plan.ticket.description)
        address = plan.get_address()
        title = _('Invitation Appointment: %s') % BeautifulSoup(plan.ticket.title).getText()

        from_email = settings.TICKET_ICS_DEFAULT_SENDER
        user, contact = utils.get_current_contact()
        organizer_full_name = ''
        if contact:
            if contact.email:
                organizer = contact.email
                from_email = '%s %s' % (contact.get_full_name(), 'contact.email')
            else:
                organizer = settings.TICKET_ICS_DEFAULT_SENDER
                from_email = organizer

            organizer_full_name = "%s %s" % (contact.get_full_name(), contact.organization.name)

#        msg = EmailMessageCalendar(title, cd['content'] or summary, from_email, recipient_list)
        msg = EmailMessageCalendar(title, summary, from_email, recipient_list)
        event = msg.get_event(dict(
            location=address,
            dtstart=plan.date_begin,
            dtend=plan.date_end,
            summary=summary,
            organizer=(organizer, organizer_full_name),
        ))
        for recipient in recipient_list:
            msg.set_attendee(event, recipient)
        msg.attach_calendar(event, 'PUBLISH')
#        msg.set_calendar(event, 'REQUEST')
        msg.send()


class ActionForm(JumboActionForm):

    contact = forms.ModelChoiceField(
        queryset=Contact.objects,
        required=False,
        widget=autocomplete_light.ChoiceWidget('Contact')
    )
    date = JumboDateField(required=False, widget=DatePicker)

    fields_map = {
        'export_action': ['output_type'],
        'close_tickets': ['contact:required', 'date'],
    }

    def clean_date(self):
        value = self.cleaned_data['date']
        return value or datetime.date.today()
