# -*- coding: utf-8 -*-
import datetime
import HTMLParser
import json

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.utils.html import strip_tags
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseForbidden
from django.template.response import TemplateResponse
from django.core import serializers
from django.contrib.admin.views.decorators import staff_member_required
from django.views.generic import TemplateView
from django.db.models import Q

from jmb.core.shortcuts import render_to_string
from jmb.jadmin import JumboModelAdmin
from jmb.ticket import models
from jmb.ticket import views as ticket_views
# from jmb.ticket import utils
# from jmb.ticket.email_communication import TicketEmail
import jmb.ticket.signals  # noqa
from jmb.jadmin import AjaxInline, register_inline

import forms as ticket_forms
from .plan import PlanAdmin, PlanAjaxInline
import fields

#########################################
#              TICKET
#########################################

class TicketAttachmentInline(admin.TabularInline):
    model = models.TicketAttachment
    extra = 0
    fields = ('file', 'description',)

class TicketAjaxInline(AjaxInline):
    model = models.Ticket
    fk_name = ''
    list_display = ('project', 'title')
    width = 1200

class TicketAdmin(JumboModelAdmin):

    class Media:
        js = ("ticket/js/admin_dajaxice.js",)

    inlines = [TicketAttachmentInline, ]
    filter_horizontal = ('assigned_to',)
    list_display_csv = (
        'get_zip_code', 'get_city', 'get_address', 'get_client', 'title',
    )
    ordering = ('project__organization', 'organization__addresses__zip_code')
    tabs = (
        ('main', {'items':
                  [
                      _('Additional Information'),
                      _('General Information'),
                      _('Note'),
                      TicketAttachmentInline,
                  ]
                  }
         ),
        ('plans', {'items': [PlanAjaxInline]}),
        # ('assistances', {'items' : [AssistancesAjaxInline]}),
    )

    def close_tickets(self, request, queryset):

        form = self.get_action_form_instance(request)
        data = form.cleaned_data

        dt = datetime.datetime.fromordinal(data['date'].toordinal())
        start = datetime.timedelta(hours=8)
        delay = datetime.timedelta(hours=1)
        for tk in queryset:
            assistance = models.TicketAssistance(
                date_begin=dt + start,
                date_end=dt + start + delay,
                ticket=tk,
                worker=data['contact'],
            )
            assistance.save()
            tk.status = 0
            tk.save()

    close_tickets.short_description = _('Close ticket & add assistance')

    actions = [close_tickets]
    action_form = ticket_forms.ActionForm
    action_form_fields = {
        'close_tickets': ['contact:required', 'date'],
    }

    def get_list_display(self, request):
        return fields.get_ticket_list_display()

    def get_readonly_fields(self, request, obj=None):
        # ADD
        if not obj:
            return fields.get_readonly_fields()
        # CHANGE
        else:
            return fields.get_readonly_fields(request.user, obj)

    def get_advanced_search_fields(self, request):
        return fields.get_advanced_search_fields(request)

    def get_fieldsets(self, request, obj=None):
        # ADD
        if not obj:
            return fields.get_ticket_fieldsets()
        # CHANGE
        else:
            return fields.get_ticket_fieldsets()

    def get_client(self, obj):
        return obj.organization.name
    get_client.short_description = _('Client')

    def get_address(self, obj):
        return obj.address.name
    get_address.short_description = _('Address')

    def get_city(self, obj):
        return obj.get_address().locality
    get_city.short_description = _('City')

    def get_zip_code(self, obj):
        return obj.address and obj.address.zip_code or ''
    get_zip_code.short_description = _('Zip Code')
    get_zip_code.admin_order_field = 'address__zip_code'

    def get_ticket_title(self, obj):
        return obj.ticket.title
    get_ticket_title.short_description = _("Title")

    def get_title(self, obj):
        return """<a class="iframe" href='%(url)s?&title=1&nobutton=1' >%(title)s</a>""" % {
            'url': '%s%s/detail/' % (reverse('admin:ticket_ticket_changelist'), obj.id),
            'title': obj.title,
        }
    get_title.allow_tags = True
    get_title.short_description = _("Title")

    def get_description_csv(self, result):
        """
        restituisce contenuto description senza TAG HTML
        """
        # rimuove TAG HTML
        description = strip_tags(result.description)

        # converte HTML in UNICODE
        _htmlparser = HTMLParser.HTMLParser()
        unescape = _htmlparser.unescape
        description = unescape(description)

        return description
    get_description_csv.short_description = _('Description')

    def get_ticket_priority_icon(self, obj):
        priority = obj.priority

        return """<div title='%(name)s' style='background-color:%(color)s;
                color:%(textcolor)s;' class='prioritydiv' >%(id)s</div>""" % {
            'name': priority.name,
            'color': priority.color,
            'textcolor': priority.color_text,
            'id': priority.priority
        }

    get_ticket_priority_icon.allow_tags = True
    get_ticket_priority_icon.short_description = _("P")
    get_ticket_priority_icon.admin_order_field = 'priority'

    def get_date_create(self, obj):
        return obj.date_create.strftime("%d/%m/%Y")
    get_date_create.short_description = _("Create")
    get_date_create.admin_order_field = 'date_create'

    def get_deadline(self, obj):
        if obj.deadline:
            not_expired = "%(deadline)s" % {
                'deadline': obj.deadline.strftime("%d/%m/%Y"),
            }
            expired = "<span class='object_error'>%(deadline)s</span>" % {
                'deadline': obj.deadline.strftime("%d/%m/%Y"),
            }
            if obj.deadline < datetime.date.today():
                return expired
            else:
                return not_expired
        else:
            return ""
    get_deadline.allow_tags = True
    get_deadline.short_description = _("Deadline")

    def get_num_assistances(self, obj):
        num_assistances = models.TicketAssistance.objects.filter(ticket=obj).count()
        if num_assistances > 0:
            if obj.status == 0:
                img = "<img src='/static/jmb/images/yes.jpg' title='Ticket Chiuso' alt='C' />"
            else:
                img = """
                    <a class='iframe hjson refresh' href='%(reverse_url)s?=1&ticket=%(ticket_id)s'>
                        <img
                            src="/static/jmb/images/add_yellow.png"
                            title="%(window_title)s"
                            style="vertical-align:middle;"
                            alt="+"
                        />
                    </a>
                """ % {
                    'reverse_url': reverse('admin:ticket_ticketassistance_add'),
                    'window_title': _("Add Assistance"),
                    'ticket_id': obj.id
                }
            return """
                %(img)s<br><a class="iframe hjson refresh"
                    href="%(reverse_url)s?ticket__in=%(ticket_id)s">
                    %(num_assistances)s
                </a>""" % {
                'reverse_url': reverse('admin:ticket_ticketassistance_changelist'),
                'window_title': _("View Ticket Assistance"),
                'num_assistances': num_assistances,
                'img': img,
                'ticket_id': obj.id
            }
        else:
            if obj.status == 0:
                return "<img src='/static/jmb/images/yes.jpg' title='Ticket Chiuso' alt='C' />"
            else:
                return """
                    <a class='iframe hjson refresh' href='%(reverse_url)s?ticket=%(ticket_id)s'>
                        <img
                            src='/static/jmb/images/work.png'
                            title="%(window_title)s" alt="+"
                        />
                    </a>""" % {
                    'reverse_url': reverse('admin:ticket_ticketassistance_add'),
                    'window_title': _("Add Ticket Assistance"),
                    'ticket_id': obj.id
                }

    get_num_assistances.short_description = 'A'
    get_num_assistances.allow_tags = True

    def get_num_comments(self, obj):
        n_comment = obj.get_comments_count()
        if n_comment:
            num_comment = n_comment
        else:
            num_comment = ""
        return """
            <a class="iframe" href="%(reverse_url)s?is_comment_list=1&nobuttons=1">
                <img
                    src='/static/jmb/images/comment.png'
                    title='Lista Commenti'
                    style='vertical-align:middle;'
                    alt='C'
                />
                <span style="
                    position: relative; bottom: 16px; left: 3px;
                    line-height: 11px;font-size:10px;
                    color: black;display: block;"
                >%(num_comments)s</span>
            </a>""" % {
            'reverse_url': reverse('admin:ticket_ticket_detail', args=(obj.id,)),
            'num_comments': num_comment
        }
    get_num_comments.short_description = 'C'
    get_num_comments.allow_tags = True

    def get_assigned_to(self, result):

        assigned_to = result.get_string_assigned_to()
        if assigned_to:
            if len(assigned_to) > 15:
                return "<span title='%s'>%s...</span>" % (assigned_to, assigned_to[:15])
            else:
                return assigned_to
        else:
            return ""
    get_assigned_to.short_description = _('Assigned To')
    get_assigned_to.allow_tags = True

    def get_urls(self):

        from django.conf.urls import patterns, url
        urls = super(TicketAdmin, self).get_urls()
        my_urls = patterns('',
                           url(
                               r'^add_comment/(?P<id>\d+)/$',
                               TemplateView.as_view(template_name='add_comment.html'),
                               name='ticket_ticket_add_comment'
                           ),
                           url(
                               r'^reopen/(?P<id>\d+)/$',
                               self.admin_site.admin_view(ticket_views.ticket_reopen),
                               name='ticket_ticket_reopen'
                           ),
                           url(
                               r'^(?P<id>\d+)/assistances/$',
                               self.admin_site.admin_view(ticket_views.ticket_assistances_list),
                               name='ticket_ticketassistances_list'
                           ),
                           url(
                               r'^calendar_list/$',
                               staff_member_required(self.calendarlist_view),
                               name='ticket_ticket_calendar_list'
                           ),
                           url(
                               r'^calendar_list/assistances/$',
                               staff_member_required(self.calendarlist_view_ass),
                               name='ticket_ticket_calendar_list_assistances'
                           ),
                           url(
                               r'^iframe_list/$',
                               self.iframe_view,
                               name='ticket_ticket_iframe_list'
                           ),
                           url(
                               r'^maintenances_from_project/(?P<project_id>\d+)/$',
                               self.maintenances_from_project,
                               name='ticket_ticket_maintenance_from_project'
                           ),
                           url(
                               r'^iframe/(?P<object_id>\d+)/$',
                               self.iframe,
                               name='ticket_ticket_iframe'
                           ),
                           url(
                               r'^workers/$',
                               self.get_workers,
                               name='app_name_model_name_workers'
                           ),
                           url(
                               r'^organization_notes/(?P<object_id>\d+)/$',
                               self.organization_notes_view,
                               name='ticket_notes_pdf'
                           ),
                           url(
                               r'^staff/$',
                               self.get_staff,
                               name='ticket_ticket_staff'
                           ),
                           )
        return my_urls + urls

    def maintenances_from_project(self, request, project_id):

        deadline = datetime.date.today() + datetime.timedelta(days=30)
        client = models.TicketProject.objects.get(pk=project_id).organization
        if not client.organization_type.category == 'cus':
            return HttpResponseForbidden("Project does not belog to a customer")
        url = reverse('admin:ticket_ticket_changelist')
        querystring = (
            'priority=20&organization={}&status__in_1=in_any&status__in_0=1&deadline__lte={}'
            '&_popup=1').format(client.id, deadline.strftime('%d/%m/%Y'))
        return HttpResponseRedirect("{}?{}".format(url, querystring))

    def get_queryset(self, request):
        qs = super(TicketAdmin, self).get_queryset(request)
        qs = qs.prefetch_related('category__translations', 'typology__translations')
        return qs

    def lookup_allowed(self, lookup, value):
        return True

    def get_workers(self, request):

        ids = request.GET.getlist('id', [])
        if not isinstance(ids, list):
            ids = [ids]
        qs = models.Contact.objects.filter(id__in=ids).all()
        data = serializers.serialize("json", qs, extras=('get_full_name',))
        return HttpResponse(data, content_type="application/json")

    def get_staff(self, request):
        qs = models.Ticket.objects.filter()
        staff_json = models.get_ticket_staff(None, True)
        staff_json = staff_json.replace(', "fields": {}', "").replace(", ", ",").replace(": ", ":").replace(',"model":"ticket.contact"', "").replace("get_full_name", "n").replace('"extras"', '"e"')
        return HttpResponse(staff_json, content_type="application/json")
    
    def iframe(self, request, object_id):
        qs = super(TicketAdmin, self).get_queryset(request).filter(id=object_id)
        data_json = serializers.serialize(
            "json", qs.all(), extras=('label', 'label_description'), use_natural_keys=True)
        return TemplateResponse(request, 'admin/ticket/iframe_callback.html',
                                {"data_json": data_json}, status=200)

    def response_change(self, request, obj):
        jpopup_param = request.GET.get('jpopup', False)
        if jpopup_param:
            return self.response_post_save_change(request, obj)
        else:
            return super(TicketAdmin, self).response_change(request, obj)
        
    def change_view(self, request, object_id, form_url='', extra_context=None):

        opts = self.model._meta
        app_label = opts.app_label

        self.change_form_template = [
            'admin/%s/%s/change_form.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_form.html' % app_label,
            'admin/change_form.html'
        ]

        if request.method == 'POST':

            jpopup_param = request.GET.get('jpopup', False)
            _json = request.POST.get('_json', False)
            patch = request.GET.get('patch', False)

            if patch:
                try:
                    request_dict = json.loads(request.body)
                    instance = models.Ticket.objects.get(id=object_id)
                except ValueError:
                    return HttpResponse('Invalid Request', status=400)
                except ObjectDoesNotExist:
                    raise Http404
                for k, v in request_dict.items():
                    setattr(instance, k, v)
                try:
                    instance.save()
                except Exception, e:
                    return HttpResponse(e, status=400)
                else:
                    json_data = serializers.serialize(
                        "json", [instance, ], extras=('label', 'label_description'))
                    return HttpResponse(json_data, content_type="application/json",
                                        status=200)

            if jpopup_param:
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.change_form_template = 'admin/ticket/change_form_iframe.html'
                response = super(TicketAdmin, self).change_view(
                    request, object_id, form_url, extra_context)
                if _json:
                    response['location'] = reverse('admin:ticket_ticket_iframe', args=[object_id])

            try:
                return response
            except UnboundLocalError:
                return super(TicketAdmin, self).change_view(
                    request, object_id, form_url, extra_context)

        if request.method == 'GET':

            json_param = request.GET.get('json', False)
            jpopup_param = request.GET.get('jpopup', False)

            if json_param:
                qs = super(TicketAdmin, self).get_queryset(request).filter(id=object_id)
                data = serializers.serialize("json", qs.all(), extras=(
                    'label', 'label_description'), use_natural_keys=True)
                return HttpResponse(data, content_type="application/json")
            if jpopup_param:
                try:
                    instance = models.Ticket.objects.get(id=object_id)
                except ObjectDoesNotExist:
                    raise Http404
                extra_context = extra_context or {}
                extra_context['is_popup'] = True
                self.change_form_template = 'admin/ticket/change_form_iframe.html'

        try:
            return super(TicketAdmin, self).change_view(
                request, object_id, form_url, extra_context)
        except ticket_forms.ForbiddenException:
            return HttpResponseForbidden(render_to_string('jmb/forbidden.html'))

    def add_view(self, request, form_url='', extra_context=None):
        opts = self.model._meta
        app_label = opts.app_label
        extra_context = extra_context or {}

        self.add_form_template = [
            'admin/%s/%s/change_form.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_form.html' % app_label,
            'admin/change_form.html'
        ]

        jpopup_param = request.GET.get('jpopup', False)

        if request.method == 'GET':
            if jpopup_param:
                extra_context['is_popup'] = True
                self.add_form_template = 'admin/ticket/change_form_iframe.html'

        if request.method == 'POST':
            if jpopup_param:
                extra_context['is_popup'] = True
                self.add_form_template = 'admin/ticket/change_form_iframe.html'

        try:
            return super(TicketAdmin, self).add_view(request, form_url, extra_context)
        except ticket_forms.ForbiddenException:
            return HttpResponseForbidden(render_to_string('jmb/forbidden.html'))

    def response_add(self, request, obj, post_url_continue=None):
        if '_json' in request.POST:
            return HttpResponseRedirect(reverse('admin:ticket_ticket_iframe', args=[obj.id]))
        return super(TicketAdmin, self).response_add(request, obj, post_url_continue)

    def calendarlist_view(self, request, extra_context=None, category=None, assistance_mode=False):
        opts = self.model._meta
        app_label = opts.app_label
        qs = super(TicketAdmin, self).get_changelist_queryset(request).order_by('priority')
        list_display = self.get_list_display(request)
        list_display_links = self.get_list_display_links(request, list_display)
        list_filter = self.get_list_filter(request)
        ChangeList = self.get_changelist(request)
        cl = ChangeList(request, self.model, list_display,
                        list_display_links, list_filter, self.date_hierarchy,
                        None, self.list_select_related, self.list_per_page,
                        self.list_max_show_all, self.list_editable, self)
        qs = cl.get_queryset(request)
        qs = qs.filter(triggerline__lte=datetime.date.today(), status=1)
        qs = qs.prefetch_related('project__staff', 'project__managers', 'project__organization__contacts',
          'typology__translations', 'category__translations')
        qs = qs.select_related(
            'project',
            'priority',
            'category',
            'typology',
            'address',
        )
        
        if request.GET.get("_json", False):
            ticket_json = serializers.serialize("json", qs, extras=('label', 'label_description'), fields=(
            'deadline', 'to_be_planned', 'public', 'status', 'project', 'priority'))
            return HttpResponse(ticket_json, content_type="application/json")
        
        today = datetime.datetime.now().date()
        tomorrow = today + datetime.timedelta(1)
        today_start = datetime.datetime.combine(today, datetime.time())
        today_end = datetime.datetime.combine(tomorrow, datetime.time())
        if(today.weekday() == 6):
            today_start = today_start + datetime.timedelta(days=1)
            today_end = today_end + datetime.timedelta(days=1)
        qsplan = models.Plan.objects.filter(date_begin__gte=today_start, date_end__lte=today_end)
        plan_json = serializers.serialize("json", qsplan.all(), extras=(
            'label',), use_natural_keys=True)
        ticketsids = [x[0] for x in qsplan.values_list("ticket")]
        qs = qs.filter(Q(to_be_planned=True) | Q(pk__in=ticketsids))
        ticket_json = serializers.serialize("json", qs, extras=('label', 'label_description'), fields=(
            'deadline', 'to_be_planned', 'public', 'status', 'project', 'priority'))
        plan_form = PlanAdmin.form
        extra_context = {
            'plan_form': plan_form,
            'ASSISTANCE_MODE': assistance_mode,
            'PLAN_INVITATION_URL':  reverse(
                settings.TICKET_PLAN_INVITATION_REVERSE_NAME, args=[999]),
            'PLAN_PDFVIEWER_URL': reverse('admin:ticket_plan_pdfviewer'),
            'PLAN_PRINT_URL':  reverse(settings.TICKET_PLAN_PRINT_REVERSE_NAME, args=[999]),
            'TICKET_CALENDAR_VIEW': settings.TICKET_CALENDAR_VIEW,
            'TICKET_HIDDEN_DAYS': settings.TICKET_HIDDEN_DAYS,
            'TICKET_SINGLE_PLAN': settings.TICKET_SINGLE_PLAN,
            'TICKETPROJECT_EDIT_URL': reverse(settings.TICKET_TICKET_REVERSE_NAME, args=[999]),
            'plan_json': plan_json.replace(', "fields": {}', "").replace(", ", ",").replace(": ", ":"),
            'ticket_json':ticket_json.replace(', "fields": {}', "").replace(", ", ",").replace(": ", ":"),
        }

        self.change_list_template = 'admin/%s/%s/calendar_list.html' % (
            app_label, opts.object_name.lower())
        response = super(TicketAdmin, self).changelist_view(request, extra_context)
        return response

    def calendarlist_view_ass(self, request, extra_context=None, category=None,
                              assistance_mode=True):
        return self.calendarlist_view(request, extra_context, category, assistance_mode)

    def iframe_view(self, request, extra_context=None, category=None):
        opts = self.model._meta
        app_label = opts.app_label

        self.change_list_template = 'admin/%s/%s/ticket_list.html' % (
            app_label, opts.object_name.lower())
        response = super(TicketAdmin, self).changelist_view(request, extra_context)
        return response

    def get_queryset(self, request):
        ids = request.GET.getlist('id', [])
        qs = super(TicketAdmin, self).get_queryset(request).distinct()
        if ids:
            if not isinstance(ids, list):
                ids = [ids, ]
            qs = models.Ticket.objects.filter(id__in=ids)
        return qs

    def changelist_view(self, request, extra_context=None):
        opts = self.model._meta
        app_label = opts.app_label
        qs = self.get_queryset(request)
        if request.is_ajax():
            data = serializers.serialize("json", qs)
            return HttpResponse(data, content_type="application/json")

        self.actions = self.actions

        self.change_list_template = [
            'admin/%s/%s/change_list.html' % (app_label, opts.object_name.lower()),
            'admin/%s/change_list.html' % app_label,
            'admin/change_list.html'
        ]

        response = super(TicketAdmin, self).changelist_view(request, extra_context)
        return response

    def get_form(self, request, obj=None, **kwargs):
        # shows add customer form
        if not obj:
            self.form = ticket_forms.TicketAdminForm

        # shows change form in function of role_ticket
        else:
            self.form = ticket_forms.TicketAdminForm
#            if self.role_ticket == "manager":
#                self.form = TicketAdminFormManager
#            elif self.role_ticket == "staff":
#                self.form = TicketAdminFormStaff
#            elif self.role_ticket == "customer":
#                self.form = TicketAdminFormCustomer
        return super(TicketAdmin, self).get_form(request, obj, **kwargs)

    def get_add_plan(self, obj):
        url = reverse('admin:ticket_plan_add') + "?ticket={}".format(obj.id)
        img = ''
        return (u'<a class="iframe hjson refresh" href="{}">'
                '<img src="{}" alt="ADD" ></a>').format(url, img)
    get_add_plan.allow_tags = True
    get_add_plan.short_description = _('P')

    def organization_notes_view(self, request, object_id):
        """
        return an HttpResponse object instance with a PDF file as attachment
        """
        obj = models.Ticket.objects.get(pk=object_id)
        return TemplateResponse(
            request, 'admin/ticket/ticket/organization_notes.html', {'obj': obj})


admin.site.register(models.Ticket, TicketAdmin)
register_inline(TicketAjaxInline)
