from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from jmb.core.shortcuts import render_to_string
from django.utils.translation import ugettext_lazy as _

class TicketEmail(object):

    def __init__(self, request, obj, created):
        self.request = request
        self.obj = obj
        self.created = created

        self.set_status_ticket()

        self.subject_msg = self.get_subject_msg()
        self.body_msg = self.html_body_msg = self.get_body_msg()
        self.from_mail = self.obj.get_from_email()
        self.recipient_list = self.get_recipient_list(obj)

    def set_status_ticket(self):
        if self.created:
            self.was_modified = False
        else:
            self.was_modified = True

        if self.obj.status == 0:
            self.is_closed = True
            self.was_reopened = False
        else:
            self.is_closed = False
            self.was_reopened = True

    def get_subject_msg(self):
        project = self.obj.project
        self.subject_msg = settings.TICKET_SUBJECT_PREFIX
        show_subject_info_project = settings.TICKET_SHOW_SUBJECT_INFO_PROJECT
        if project and show_subject_info_project:
            self.subject_msg = "%s[#%s][%s]" % (
                settings.TICKET_SUBJECT_PREFIX, project.id, self.obj.organization)
        if self.is_closed:
            self.subject_msg += "[CHIUSO] - #%s - %s" % (self.obj.id, self.obj.title)
        elif self.was_reopened:
            self.subject_msg += "[RIAPERTO] - #%s - %s" % (self.obj.id, self.obj.title)
        elif self.was_modified:
            self.subject_msg += "[MODIFICATO] - #%s - %s" % (self.obj.id, self.obj.title)
        else:
            self.subject_msg += "[APERTO] - #%s - %s" % (self.obj.id, self.obj.title)
        return self.subject_msg

    def get_body_msg(self):
        self.body_msg = render_to_string("ticket/email/ticket_info_base.html",
                                         ticket=self.obj,
                                         diffs=self.get_diffs(),
                                         status_ticket=self.get_status_ticket())
        return self.body_msg

    def get_diffs(self):
        """return a list of tuples of differences between old and new instance
        Each tuple has an attribute and a text
        """
        diffs = []
        new = self.obj
        old = self.obj._old_obj
        if not old:
            return diffs
        for attr in new._meta.fields:
            verbose_name = attr.verbose_name
            name = attr.name
            if name.startswith('_') or name in ('date_create', 'submitter',
                                                'last_modifier', 'date_last_modify', 'to_be_planned'):
                continue
            if not getattr(old, name) == getattr(new, name):
                new_value = unicode(getattr(new, name))
                old_value = unicode(getattr(old, name))
                try:
                    if attr.choices:
                        new_value = dict(attr.choices)[int(new_value)]
                        old_value = dict(attr.choices)[int(old_value)]
                except AttributeError:
                    pass
                diffs.append([_(verbose_name) or _(name), old_value, new_value, unicode(name)])
        return diffs

    def get_recipient_list(self, ticket):
        "Return a list of recipients for this email"

        project = ticket.project or False

        recipients = set()
        recipients.update(ticket.assigned_to.all())
        recipients.add(ticket.assigned_by)
        if project:
            recipients.update(ticket.project.managers.all())
            if ticket.public:
                recipients.update(ticket.project.customers.all())

        return [c.get_email() for c in recipients if c and c.get_email()]

    def get_status_ticket(self):
        if self.created:
            return "inserito"
        else:
            return "modificato"
        if self.obj.status == 0:
            return "chiuso"
        else:
            return "riaperto"

    def send(self):
        # submitter = None
        # if self.obj.submitter and self.obj.submitter.email:
        #     submitter = [self.obj.submitter.email or settings.TICKET_SEND_EMAIL]

        for dest in self.recipient_list:
            msg = EmailMultiAlternatives(self.subject_msg,
                                         self.body_msg,
                                         self.from_mail,  # FIXME usiamo ancora get_from_email?
                                         to=[dest],  # FIXME: prima era bss a tutti
                                         )
            msg.attach_alternative(self.html_body_msg, "text/html")
            msg.send()


class TicketCloseMail(TicketEmail):

    def __init__(self, ticket):
        self.is_closed = False
        self.obj = ticket
        self.subject_msg = "%s[CHIUSO] - #%s - %s" % (
            settings.TICKET_SUBJECT_PREFIX, ticket.id, ticket.title)
        if self.obj.project:
            self.subject_msg = "%s[#%s][CHIUSO] - #%s - %s" % (
                settings.TICKET_SUBJECT_PREFIX, ticket.project.id, ticket.id, ticket.title)
        self.body_msg = self.html_body_msg = self.get_body_msg()
        self.from_mail = ticket.get_from_email()
        self.recipient_list = self.get_recipient_list()

    def get_body_msg(self):
        self.body_msg = render_to_string("ticket/email/ticket_info_base.html",
                                         ticket=self.obj,
                                         status_ticket='chiuso')
        return self.body_msg

    def get_recipient_list(self):
        project = self.obj.project
        assigned_to = self.obj.assigned_to.all()

        return self._get_assegnatari(project, assigned_to)


class TicketEmailNonAssegnato(TicketEmail):

    def __init__(self, request, obj, form, change):
        super(TicketEmailNonAssegnato, self).__init__(request, obj, form, change)

    def get_recipient_list(self):
        return settings.TICKET_MAIL_ADMIN

    def get_subject_msg(self):
        self.subject_msg = "%s[NON ASSEGNATO] - #%s - %s" % (
            settings.TICKET_SUBJECT_PREFIX, self.t.id, self.t.title)
        return self.subject_msg

    def get_body_msg(self):
        self.body_msg = render_to_string("ticket/email/ticket_info_noassegnatari.html",
                                         ticket=self.t,
                                         status_ticket=self.get_status_ticket())
        return self.body_msg
